@@ -1,9 +1,9 @@
 //
-// im_scim_1.2.cpp - SCIM plugin for mlterm (c++ part)
+// im_scim_1.4.cpp - SCIM plugin for mlterm (c++ part)
 //
 // Copyright (C) 2005 Seiichi SATO <ssato@sh.rim.or.jp>
 //
-//	$Id: im_scim_1.2.cpp,v 1.3 2005/03/03 15:34:13 seiichi Exp $
+//	$Id: im_scim_1.4.cpp,v 1.3 2007/11/12 17:15:07 mfabian Exp $
 //
 
 // This file is partially based on gtkimcontextscim.cpp of SCIM
@@ -30,6 +30,7 @@
 #include  <iostream>
 #include  <stdio.h>
 #include  <X11/Xlib.h>  // KeySym, XKeyEvent
+#include  <string.h>
 
 #include  "im_scim.h"
 
@@ -45,11 +46,6 @@
 
 #define  TIMEOUT  5000	// msec
 
-const int  KEY_TRIGGER = 1 ;
-const int  KEY_MENU = 2 ;
-const int  KEY_NEXT_FACTORY = 3 ;
-const int  KEY_PRE_FACTORY = 4 ;
-
 using namespace scim ;
 
 typedef struct  im_scim_context_private
@@ -70,19 +66,6 @@ typedef struct  im_scim_context_private
 
 }  im_scim_context_private_t ;
 
-#define transaction_init(id)				\
-do {							\
-	int  cmd ;					\
-	uint32  data ;					\
-	sock.clear() ;					\
-	sock.put_command( SCIM_TRANS_CMD_REQUEST) ;	\
-	sock.put_data( panel_key) ;			\
-	sock.put_data( (uint32)(id)) ;			\
-	sock.get_command( cmd) ;			\
-	sock.get_data( data) ;				\
-	sock.get_data( data) ;				\
-}  while(0)
-
 #define C_STR(s) (char*) ( utf8_wcstombs((s)).c_str())
 
 // --- static variables ---
@@ -96,41 +79,19 @@ static ConfigPointer  config = NULL ;
 
 static BackEndPointer  be = NULL ;
 
-static HotkeyMatcher  keymatcher_frontend ;
+static FrontEndHotkeyMatcher  keymatcher_frontend ;
 static IMEngineHotkeyMatcher  keymatcher_imengine ;
 
-static SocketClient  panel ;
-static uint32  panel_key ;
-
-static Transaction  sock ;
+static PanelClient  panel_client ;
 
 static bool  is_vertical_lookup ;
+static int  valid_key_mask = 0 ;
 
 static int  id = 0 ; /* XXX */
 
+
 // --- static functions ---
 
-static void
-load_config( const ConfigPointer &  config)
-{
-	static KeyEventList  key ;
-
-	scim_string_to_key_list( key , config->read( String( SCIM_CONFIG_HOTKEYS_FRONTEND_TRIGGER) , String( "Control+space"))) ;
-	keymatcher_frontend.add_hotkeys( key , KEY_TRIGGER) ;
-
-	scim_string_to_key_list( key , config->read( String( SCIM_CONFIG_HOTKEYS_FRONTEND_SHOW_FACTORY_MENU) , String( "Control+Alt+l,Control+Alt+m,Control+Alt+s,Control+Alt+Right"))) ;
-	keymatcher_frontend.add_hotkeys( key , KEY_MENU) ;
-
-	scim_string_to_key_list( key , config->read( String( SCIM_CONFIG_HOTKEYS_FRONTEND_NEXT_FACTORY) , String( "Control+Alt+Down,Control+Shift_R,Control+Shift_L"))) ;
-	keymatcher_frontend.add_hotkeys( key , KEY_NEXT_FACTORY) ;
-
-	scim_string_to_key_list( key , config->read( String( SCIM_CONFIG_HOTKEYS_FRONTEND_PREVIOUS_FACTORY) , String( "Control+Alt+Up,Shift+Control_R,Shift+Control_L"))) ;
-	keymatcher_frontend.add_hotkeys( key , KEY_PRE_FACTORY) ;
-
-	// hack
-	is_vertical_lookup = config->read( String( "/Panel/Gtk/LookupTableVertical") , false) ;
-}
-
 static im_scim_context_private_t *
 id_to_context(
 	int  id
@@ -150,55 +111,32 @@ id_to_context(
 }
 
 
-static void
-send_factory_menu_item(
-	im_scim_context_private_t *  context
-	)
-{
-	std::vector<IMEngineFactoryPointer>  factories ;
-	size_t  i ;
-
-	be->get_factories_for_encoding( factories , "UTF-8") ;
-
-	transaction_init( context->id) ;
-	sock.put_command( SCIM_TRANS_CMD_PANEL_SHOW_FACTORY_MENU) ;
-	for( i = 0 ; i < factories.size() ; i++)
-	{
-		sock.put_data( factories[i]->get_uuid()) ;
-		sock.put_data( utf8_wcstombs( factories[i]->get_name()));
-		sock.put_data( factories[i]->get_language());
-		sock.put_data( factories[i]->get_icon_file());
-	}
-	sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
-}
+//
+// callback for config modules
+//
 
 static void
-send_help_description(
-	im_scim_context_private_t *  context
-	)
+cb_config_load( const ConfigPointer &  config)
 {
-	String  desc ;
-	String  str ;
+	KeyEvent  key ;
 
-	desc += utf8_wcstombs( context->factory->get_name()) +
-		String( ":\n\n");
-	desc += utf8_wcstombs( context->factory->get_authors()) +
-		String( "\n\n");
-	desc += String( "  Help:\n    ") +
-		utf8_wcstombs( context->factory->get_help()) +
-		String( "\n\n");
-	desc += utf8_wcstombs( context->factory->get_credits()) +
-		String( "\n\n");
+	keymatcher_frontend.load_hotkeys( config);
+	keymatcher_imengine.load_hotkeys( config);
 
-	transaction_init( context->id) ;
-	sock.put_command( SCIM_TRANS_CMD_PANEL_SHOW_HELP);
-	sock.put_data( desc) ;
-	sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
+	scim_string_to_key( key, config->read( String( SCIM_CONFIG_HOTKEYS_FRONTEND_VALID_KEY_MASK) , String( "Shift+Control+Alt+Lock")));
+
+	valid_key_mask = key.mask > 0 ? key.mask : 0xffff ;
+	valid_key_mask |= SCIM_KEY_ReleaseMask ;
+
+	scim_global_config_flush();
+
+	// hack
+	is_vertical_lookup = config->read( String( "/Panel/Gtk/LookupTableVertical") , false) ;
 }
 
 
 //
-// callbacks from backend
+// callbacks for backend
 //
 
 static void
@@ -211,20 +149,13 @@ cb_commit(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && context->on)
 	{
-		return ;
+		context->preedit_attr.clear() ;
+
+		(*context->cb->commit)( context->self , C_STR( wstr)) ;
+		(*context->cb->candidate_hide)( context->self) ;
 	}
-
-	if( ! context->on)
-	{
-		return ;
-	}
-
-	context->preedit_attr.clear() ;
-
-	(*context->cb->commit)( context->self , C_STR( wstr)) ;
-	(*context->cb->candidate_hide)( context->self) ;
 }
 
 static void
@@ -238,18 +169,11 @@ cb_preedit_update(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && context->on)
 	{
-		return ;
+		context->preedit_str = wstr ;
+		context->preedit_attr = attr ;
 	}
-
-	if( ! context->on)
-	{
-		return ;
-	}
-
-	context->preedit_str = wstr ;
-	context->preedit_attr = attr ;
 }
 
 static void
@@ -261,20 +185,13 @@ cb_preedit_hide(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && context->on)
 	{
-		return ;
+		context->preedit_str = WideString() ;
+		context->preedit_attr.clear() ;
+
+		(*context->cb->preedit_update)( context->self , NULL , 0) ;
 	}
-
-	if( ! context->on)
-	{
-		return ;
-	}
-
-	context->preedit_str = WideString() ;
-	context->preedit_attr.clear() ;
-
-	(*context->cb->preedit_update)( context->self , NULL , 0) ;
 }
 
 static void
@@ -287,21 +204,14 @@ cb_preedit_caret(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && context->on)
 	{
-		return ;
+		context->preedit_caret = caret ;
+
+		(*context->cb->preedit_update)( context->self ,
+						C_STR( context->preedit_str) ,
+						caret) ;
 	}
-
-	if( ! context->on)
-	{
-		return ;
-	}
-
-	context->preedit_caret = caret ;
-
-	(*context->cb->preedit_update)( context->self ,
-					C_STR( context->preedit_str) ,
-					caret) ;
 }
 
 static void
@@ -360,17 +270,10 @@ cb_lookup_show(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && context->on)
 	{
-		return ;
+		(*context->cb->candidate_show)( context->self) ;
 	}
-
-	if( ! context->on)
-	{
-		return ;
-	}
-
-	(*context->cb->candidate_show)( context->self) ;
 }
 
 static void
@@ -382,17 +285,10 @@ cb_lookup_hide(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && context->on)
 	{
-		return ;
+		(*context->cb->candidate_hide)( context->self) ;
 	}
-
-	if( ! context->on)
-	{
-		return ;
-	}
-
-	(*context->cb->candidate_hide)( context->self) ;
 }
 
 static void
@@ -405,20 +301,12 @@ cb_prop_register(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && panel_client.is_connected())
 	{
-		return ;
+		panel_client.prepare( context->id) ;
+		panel_client.register_properties( context->id , props) ;
+		panel_client.send() ;
 	}
-
-	if( ! panel.is_connected())
-	{
-		return ;
-	}
-
-	transaction_init( context->id) ;
-	sock.put_command( SCIM_TRANS_CMD_REGISTER_PROPERTIES) ;
-	sock.put_data( props) ;
-	sock.write_to_socket( panel , SCIM_TRANS_MAGIC);
 }
 
 static void
@@ -431,20 +319,12 @@ cb_prop_update(
 
 	context = static_cast <im_scim_context_private_t *> (instance->get_frontend_data()) ;
 
-	if( ! context)
+	if( context && panel_client.is_connected())
 	{
-		return ;
+		panel_client.prepare( context->id) ;
+		panel_client.update_property( context->id , prop) ;
+		panel_client.send() ;
 	}
-
-	if( ! panel.is_connected())
-	{
-		return ;
-	}
-
-	transaction_init( context->id) ;
-	sock.put_command( SCIM_TRANS_CMD_UPDATE_PROPERTY) ;
-	sock.put_data( prop) ;
-	sock.write_to_socket( panel , SCIM_TRANS_MAGIC);
 }
 
 static void
@@ -453,12 +333,16 @@ set_callbacks(
 	)
 {
 	context->instance->signal_connect_commit_string( slot( cb_commit)) ;
+
 	context->instance->signal_connect_update_preedit_string( slot( cb_preedit_update)) ;
 	context->instance->signal_connect_hide_preedit_string( slot( cb_preedit_hide)) ;
+
 	context->instance->signal_connect_update_preedit_caret( slot( cb_preedit_caret)) ;
+
 	context->instance->signal_connect_update_lookup_table( slot( cb_lookup_update)) ;
 	context->instance->signal_connect_show_lookup_table( slot( cb_lookup_show)) ;
 	context->instance->signal_connect_hide_lookup_table( slot( cb_lookup_hide)) ;
+
 	context->instance->signal_connect_register_properties( slot( cb_prop_register)) ;
 	context->instance->signal_connect_update_property( slot( cb_prop_update)) ;
 
@@ -466,26 +350,142 @@ set_callbacks(
 }
 
 
+//
+// callbacks for panel clients
+//
+
+static void
+cb_panel_request_factory_menu(
+	int  id
+	)
+{
+	im_scim_context_private_t *  context ;
+	std::vector<IMEngineFactoryPointer>  factories ;
+	std::vector<PanelFactoryInfo>  menu;
+	size_t  i ;
+
+	context = id_to_context( id) ;
+
+	be->get_factories_for_encoding( factories , "UTF-8") ;
+	for( i = 0 ; i < factories.size() ; i++)
+	{
+		menu.push_back(PanelFactoryInfo(
+				factories[i]->get_uuid() ,
+				utf8_wcstombs( factories[i]->get_name()) ,
+				factories[i]->get_language() ,
+				factories[i]->get_icon_file()));
+	}
+
+	panel_client.prepare( id) ;
+	panel_client.show_factory_menu( id , menu) ;
+	panel_client.send() ;
+}
+
+static void
+cb_panel_request_help(
+	int  id
+	)
+{
+	im_scim_context_private_t *  context ;
+	String  desc ;
+	String  str ;
+
+	context = id_to_context( id) ;
+
+	desc += utf8_wcstombs( context->factory->get_name()) +
+		String( ":\n\n");
+	desc += utf8_wcstombs( context->factory->get_authors()) +
+		String( "\n\n");
+	desc += String( "  Help:\n    ") +
+		utf8_wcstombs( context->factory->get_help()) +
+		String( "\n\n");
+	desc += utf8_wcstombs( context->factory->get_credits()) +
+		String( "\n\n");
+
+	panel_client.prepare( id) ;
+	panel_client.show_help( id , desc) ;
+	panel_client.send() ;
+}
+
+static void
+cb_panel_change_factory(
+	int  id ,
+	const String &  uuid
+	)
+{
+	im_scim_context_private_t *  context ;
+	IMEngineFactoryPointer  factory ;
+	PanelFactoryInfo  info ;
+
+	context = id_to_context( id) ;
+
+	factory = be->get_factory( uuid) ;
+	if( factory.null())
+	{
+		return ;
+	}
+
+	panel_client.prepare( id) ;
+
+	if( uuid.length() == 0)
+	{
+		panel_client.turn_off( id) ;
+		panel_client.focus_out( id) ;
+		panel_client.send() ;
+		context->on = 0 ;
+		return ;
+	}
+
+	context->factory = factory ;
+	context->instance->focus_out() ;
+	be->set_default_factory( lang , context->factory->get_uuid()) ;
+	context->instance = context->factory->create_instance( String( "UTF-8") , context->id) ;
+	set_callbacks( context) ;
+	info = PanelFactoryInfo(context->factory->get_uuid() ,
+				utf8_wcstombs( context->factory->get_name()) ,
+				context->factory->get_language() ,
+				context->factory->get_icon_file()) ;
+	panel_client.update_factory_info( id, info) ;
+	panel_client.send() ;
+	context->instance->focus_in() ;
+}
+
+static void
+cb_panel_trigger_property(
+	int  id ,
+	const String &  prop
+	)
+{
+	im_scim_context_private_t *  context ;
+
+	context = id_to_context( id) ;
+
+	panel_client.prepare( id) ;
+	context->instance->trigger_property( prop);
+	panel_client.send() ;
+}
+
+
 // --- global functions ---
 
 int
 im_scim_initialize( char *  locale)
 {
-	SocketAddress  sock_addr;
-	SocketClient  sock_client;
-	uint32  key ;
+	SocketAddress  address;
+	SocketClient  client;
+	uint32  magic ;
 	std::vector<String>  imengines ;
 	std::vector<String>  config_modules ;
 	String  config_mod_name ;
 
 	lang = scim_get_locale_language( String( locale)) ;
 
-	sock_addr.set_address( scim_get_default_socket_frontend_address());
-	if( ! sock_client.connect(sock_addr) &&
-	    ! scim_socket_open_connection( key ,
+	address.set_address( scim_get_default_socket_frontend_address());
+	if( ! client.connect(address) &&
+	    ! scim_socket_open_connection( magic ,
 					   String( "ConnectionTester") ,
 					   String( "SocketFrontEnd") ,
-					   sock_client ,
+					   client ,
 					   TIMEOUT))
 	{
 		kik_error_printf( "Unable to connect to the socket frontend.\n") ;
@@ -533,7 +533,7 @@ im_scim_initialize( char *  locale)
 		goto  error ;
 	}
 
-	config = config_module->create_config( "scim") ;
+	config = config_module->create_config() ;
 
 	if( config.null())
 	{
@@ -549,21 +549,19 @@ im_scim_initialize( char *  locale)
 		goto  error ;
 	}
 
-	sock_addr.set_address( scim_get_default_panel_socket_address( getenv( "DISPLAY"))) ; // FIXME
-	if( panel.connect( sock_addr))
-	{
-		if( ! scim_socket_open_connection( panel_key ,
-						   String( "FrontEnd") ,
-						   String( "Panel") ,
-						   panel ,
-						   TIMEOUT))
-		{
-			panel.close() ;
-		}
-	}
+	cb_config_load( config) ;
+	config->signal_connect_reload( slot(cb_config_load)) ;
 
-	load_config( config) ;
-	config->signal_connect_reload( slot(load_config)) ;
+	panel_client.signal_connect_request_factory_menu( slot (cb_panel_request_factory_menu));
+	panel_client.signal_connect_request_help( slot (cb_panel_request_help)) ;
+	panel_client.signal_connect_change_factory( slot (cb_panel_change_factory)) ;
+
+	panel_client.signal_connect_trigger_property( slot (cb_panel_trigger_property)) ;
+
+	if (panel_client.open_connection( config->get_name() , getenv( "DISPLAY")) == false)
+	{
+		goto  error;
+	}
 
 	context_table.clear() ;
 
@@ -581,9 +579,9 @@ error:
 		be.reset() ;
 	}
 
-	if( panel.is_connected())
+	if( panel_client.is_connected())
 	{
-		panel.close() ;
+		panel_client.close_connection() ;
 	}
 
 	return  0 ;
@@ -592,9 +590,9 @@ error:
 int
 im_scim_finalize( void)
 {
-	if( panel.is_connected())
+	if( panel_client.is_connected())
 	{
-		panel.close() ;
+		panel_client.close_connection() ;
 	}
 
 	if( ! be.null())
@@ -627,8 +625,7 @@ im_scim_create_context(
 
 	context->factory = be->get_default_factory( lang , String( "UTF-8")) ;
 
-	if( ! ( context->instance = context->factory->create_instance( String( "UTF-8") ,
-							      id)))
+	if( ! ( context->instance = context->factory->create_instance( String( "UTF-8") , id)))
 	{
 		kik_error_printf( "Could not create new instance.\n") ;
 		return  NULL ;
@@ -678,25 +675,27 @@ im_scim_focused(
 
 	context = (im_scim_context_private_t *) _context ;
 
-	if( panel.is_connected())
+	if( panel_client.is_connected())
 	{
-		transaction_init( context->id) ;
-		sock.put_command( SCIM_TRANS_CMD_FOCUS_IN) ;
-		sock.put_data( context->instance->get_factory_uuid()) ;
+		panel_client.prepare( context->id) ;
+		panel_client.focus_in( context->id ,
+				       context->instance->get_factory_uuid()) ;
 		if( context->on)
 		{
-			sock.put_command( SCIM_TRANS_CMD_PANEL_UPDATE_FACTORY_INFO);
-			sock.put_data( context->factory->get_uuid()) ;
-			sock.put_data( utf8_wcstombs( context->factory->get_name()));
-			sock.put_data( context->factory->get_language()) ;
-			sock.put_data( context->factory->get_icon_file()) ;
-			sock.put_command( SCIM_TRANS_CMD_PANEL_TURN_ON) ;
+			PanelFactoryInfo  info ;
+			info = PanelFactoryInfo(
+				context->factory->get_uuid() ,
+				utf8_wcstombs( context->factory->get_name()) ,
+				context->factory->get_language() ,
+				context->factory->get_icon_file()) ;
+			panel_client.update_factory_info( context->id, info) ;
+			panel_client.turn_on( context->id) ;
 		}
 		else
 		{
-			sock.put_command( SCIM_TRANS_CMD_PANEL_TURN_OFF) ;
+			panel_client.turn_off( context->id) ;
 		}
-		sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
+		panel_client.send() ;
 	}
 
 	context->instance->focus_in() ;
@@ -717,13 +716,12 @@ im_scim_unfocused(
 
 	context = (im_scim_context_private_t *) _context ;
 
-	if( panel.is_connected())
+	if( panel_client.is_connected())
 	{
-		transaction_init( context->id) ;
-		sock.put_command( SCIM_TRANS_CMD_FOCUS_IN) ;
-		sock.put_command( SCIM_TRANS_CMD_PANEL_TURN_OFF);
-		sock.put_command( SCIM_TRANS_CMD_FOCUS_OUT) ;
-		sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
+		panel_client.prepare( context->id) ;
+		panel_client.turn_off( context->id) ;
+		panel_client.focus_in( context->id , context->instance->get_factory_uuid ()) ;
+		panel_client.send() ;
 	}
 
 	context->instance->focus_out() ;
@@ -743,70 +741,91 @@ im_scim_key_event(
 	)
 {
 	im_scim_context_private_t *  context ;
+	FrontEndHotkeyAction  hotkey_action ;
 	int  key_type ;
 	KeyEvent  scim_key ;
+	PanelFactoryInfo  info ;
 
 	context = (im_scim_context_private_t *) _context ;
 
-	scim_key.mask = event->state & ~SCIM_KEY_ReleaseMask;
+	scim_key.mask = event->state & valid_key_mask;
 	scim_key.code = ksym ;
 
 	keymatcher_frontend.push_key_event( scim_key) ;
 	keymatcher_imengine.push_key_event( scim_key) ;
 
-	key_type = keymatcher_frontend.get_match_result() ;
+	hotkey_action = keymatcher_frontend.get_match_result() ;
 
-	switch( key_type)
+	if( hotkey_action == SCIM_FRONTEND_HOTKEY_OFF && ! context->on)
+	{
+		return  0 ;
+	}
+
+	if( hotkey_action == SCIM_FRONTEND_HOTKEY_ON && context->on)
+	{
+		return  0 ;
+	}
+
+	if( hotkey_action == SCIM_FRONTEND_HOTKEY_TRIGGER)
 	{
-	case KEY_TRIGGER:
-		transaction_init( context->id) ;
-		sock.put_command( SCIM_TRANS_CMD_FOCUS_IN) ;
-		sock.put_data( context->instance->get_factory_uuid()) ;
 		if( context->on)
 		{
-			if( panel.is_connected())
-			{
-				sock.put_command( SCIM_TRANS_CMD_PANEL_TURN_OFF) ;
-				sock.put_command( SCIM_TRANS_CMD_FOCUS_OUT) ;
-				sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
-			}
-
-			(*context->cb->preedit_update)( context->self ,
-							NULL , 0) ;
-			(*context->cb->candidate_hide)( context->self) ;
-
-			context->instance->focus_out() ;
-
-			context->on = 0 ;
+			hotkey_action = SCIM_FRONTEND_HOTKEY_OFF ;
 		}
 		else
 		{
-			if( panel.is_connected())
-			{
-				sock.put_command( SCIM_TRANS_CMD_PANEL_UPDATE_FACTORY_INFO);
-				sock.put_data( context->factory->get_uuid()) ;
-				sock.put_data( utf8_wcstombs( context->factory->get_name())) ;
-				sock.put_data( context->factory->get_language()) ;
-				sock.put_data( context->factory->get_icon_file()) ;
-				sock.put_command( SCIM_TRANS_CMD_PANEL_TURN_ON) ;
-				sock.put_command( SCIM_TRANS_CMD_FOCUS_IN) ;
-				sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
-			}
+			hotkey_action = SCIM_FRONTEND_HOTKEY_ON ;
+		}
+	}
 
-			(*context->cb->preedit_update)(
-						context->self ,
+	info = PanelFactoryInfo( context->factory->get_uuid() ,
+				 utf8_wcstombs( context->factory->get_name()) ,
+				 context->factory->get_language() ,
+				 context->factory->get_icon_file()) ;
+
+	switch( hotkey_action)
+	{
+	case SCIM_FRONTEND_HOTKEY_ON:
+		if( panel_client.is_connected())
+		{
+			panel_client.prepare( context->id) ;
+			panel_client.update_factory_info( context->id , info) ;
+			panel_client.turn_on( context->id) ;
+			panel_client.focus_in( context->id , context->instance->get_factory_uuid ()) ;
+			panel_client.send() ;
+		}
+
+		(*context->cb->preedit_update)( context->self ,
 						C_STR( context->preedit_str) ,
 						context->preedit_caret) ;
-			(*context->cb->candidate_show)( context->self) ;
+		(*context->cb->candidate_show)( context->self) ;
 
-			context->instance->focus_in() ;
+		context->instance->focus_in() ;
+
+		context->on = 1 ;
 
-			context->on = 1 ;
-		}
 		return  0 ;
-	case KEY_MENU:
-	case KEY_NEXT_FACTORY:
-	case KEY_PRE_FACTORY:
+	case SCIM_FRONTEND_HOTKEY_OFF:
+		if( panel_client.is_connected())
+		{
+			panel_client.prepare( context->id) ;
+			panel_client.turn_off( context->id) ;
+			panel_client.focus_out( context->id) ;
+			panel_client.send() ;
+		}
+
+		(*context->cb->preedit_update)( context->self ,
+						NULL , 0) ;
+		(*context->cb->candidate_hide)( context->self) ;
+
+		context->instance->focus_out() ;
+
+		context->on = 0 ;
+
+		return  0 ;
+	case SCIM_FRONTEND_HOTKEY_NEXT_FACTORY:
+	case SCIM_FRONTEND_HOTKEY_PREVIOUS_FACTORY:
+	case SCIM_FRONTEND_HOTKEY_SHOW_FACTORY_MENU:
 		// not implemented yet
 		return  0 ;
 	default:
@@ -820,7 +839,6 @@ im_scim_key_event(
 
 	if( context->instance->process_key_event( scim_key))
 	{
-		sock.write_to_socket( panel , SCIM_TRANS_MAGIC);
 		return  0 ;
 	}
 
@@ -888,9 +906,9 @@ im_scim_preedit_char_attr(
 int
 im_scim_get_panel_fd( void)
 {
-	if( panel.is_connected())
+	if( panel_client.is_connected())
 	{
-		return  panel.get_id() ;
+		return  panel_client.get_connection_number() ;
 	}
 
 	return  -1 ;
@@ -906,86 +924,7 @@ im_scim_receive_panel_event( void)
 	uint32  data_uint32 ;
 	int  id ;
 
-	if( ! sock.read_from_socket( panel , TIMEOUT))
-	{
-		// XXX
-		return  0 ;
-	}
-
-	sock.get_command( command) ;
-	if( command != SCIM_TRANS_CMD_REPLY)
-	{
-		return  1 ;
-	}
-
-	type = sock.get_data_type() ;
-	if( type == SCIM_TRANS_DATA_COMMAND)
-	{
-		while( sock.get_command(command))
-		{
-			if( command == SCIM_TRANS_CMD_RELOAD_CONFIG)
-			{
-				config->reload() ;
-				load_config( config) ;
-			}
-			else if( command == SCIM_TRANS_CMD_EXIT)
-			{
-				// not implemented yet
-			}
-		}
-
-		return  1 ;
-	}
-
-	sock.get_data( data_uint32) ;
-	id = (int) data_uint32 ;
-
-	context = id_to_context( id) ;
-
-	sock.get_command( command) ;
-	switch( command)
-	{
-	case SCIM_TRANS_CMD_PANEL_REQUEST_FACTORY_MENU:
-		send_factory_menu_item( context) ;
-		break ;
-	case SCIM_TRANS_CMD_PANEL_CHANGE_FACTORY:
-		sock.get_data( string) ;
-		transaction_init( context->id) ;
-		sock.put_command( SCIM_TRANS_CMD_FOCUS_IN) ;
-		sock.put_data( context->instance->get_factory_uuid()) ;
-		if( string.length() == 0)
-		{
-			sock.put_command( SCIM_TRANS_CMD_PANEL_TURN_OFF) ;
-			sock.put_command( SCIM_TRANS_CMD_FOCUS_OUT) ;
-			sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
-			context->on = 0 ;
-			return  1 ;
-		}
-		context->factory = be->get_factory( string) ;
-		context->instance->focus_out() ;
-		be->set_default_factory( lang , context->factory->get_uuid()) ;
-		context->instance = context->factory->create_instance( String( "UTF-8") , context->id) ;
-		set_callbacks( context) ;
-		sock.put_command( SCIM_TRANS_CMD_PANEL_UPDATE_FACTORY_INFO);
-		sock.put_data( context->factory->get_uuid());
-		sock.put_data( utf8_wcstombs(context->factory->get_name()));
-		sock.put_data( context->factory->get_language()) ;
-		sock.put_data( context->factory->get_icon_file()) ;
-		sock.write_to_socket(panel , SCIM_TRANS_MAGIC);
-		context->instance->focus_in() ;
-
-		break ;
-	case SCIM_TRANS_CMD_PANEL_REQUEST_HELP:
-		send_help_description( context) ;
-		break ;
-	case SCIM_TRANS_CMD_TRIGGER_PROPERTY:
-		sock.get_data( string) ;
-		context->instance->trigger_property( string) ;
-		break;
-	default:
-		break ;
-	}
-
+	panel_client.filter_event() ;
 	return  1 ;
 }
 
