@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_file.h,v 1.5 2003/01/12 12:34:31 arakiken Exp $
+ *	$Id: kik_file.h,v 1.6 2006/10/19 14:41:51 h_minami Exp $
  */
 
 #ifndef  __KIK_FILE_H__
@@ -33,5 +33,8 @@ int  kik_file_lock( int  fd) ;
 
 int  kik_file_unlock( int  fd) ;
 
+int  kik_file_set_cloexec( int  fd) ;
+
+int  kik_file_unset_cloexec( int  fd) ;
 
 #endif
