@@ -1,5 +1,5 @@
 /*
- *	$Id: kik_pty_bsd.c,v 1.9 2003/03/08 03:19:29 arakiken Exp $
+ *	$Id: kik_pty_bsd.c,v 1.10 2006/10/19 14:41:51 h_minami Exp $
  */
 
 #include  "kik_pty.h"
@@ -118,6 +118,7 @@ open_pty(
 			}
 			else
 			{
+				kik_file_set_cloexec( *master) ;
 				/*
 				 * we succeeded to open pty master.
 				 * opening pty slave in succession. 
@@ -136,6 +137,7 @@ open_pty(
 					}
 					else
 					{
+						kik_file_set_cloexec( *slave) ;
 						return  1 ;
 					}
 				}
@@ -195,7 +197,6 @@ kik_pty_fork(
 	/*
 	 * parent process
 	 */
-
 	/*
 	 * delaying.
 	 */
@@ -301,5 +302,7 @@ kik_pty_fork(
 	#endif
 	}
 	
+	kik_file_set_cloexec( *slave) ;	
+	
 	return  pid ;
 }
