@@ -1,12 +1,13 @@
 /*
- *	$Id: kik_utmp_utmper.c,v 1.1 2002/02/13 14:42:55 arakiken Exp $
+ *	$Id: kik_utmp_utmper.c,v 1.2 2005/09/23 05:40:20 mfabian Exp $
  */
 
 #include  "kik_utmp.h"
 
 #include  <stdio.h>	/* NULL */
+#include  <utempter.h>
 #include  "kik_mem.h"		/* malloc/free */
-
+#include  "kik_str.h"		/* strdup */
 
 struct  kik_utmp
 {
@@ -30,13 +31,13 @@ kik_utmp_new(
 	{
 		return  NULL ;
 	}
-		
-	if( ( utmp->tty = (char *) malloc( strlen( tty ) + 1)) == NULL)
+
+	if( ( utmp->tty = strdup( tty )) == NULL)
 	{
+		free( utmp) ;
 		return  NULL ;
 	}
 
-	strcpy(utmp->tty, tty);
 	utmp->fd = pty_fd;
 
 	addToUtmp(tty, host, pty_fd);
