@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_char_encoding.h,v 1.3 2003/04/06 10:40:36 arakiken Exp $
+ *	$Id: ml_char_encoding.h,v 1.5 2004/12/07 03:37:00 h_minami Exp $
  */
 
 #ifndef  __ML_CHAR_ENCODING_H__
@@ -12,8 +12,8 @@
 
 
 /*
- * supported encodings are those which are not conflicted with US_ASCII.
- * so , UCS4 is not supported.
+ * Supported encodings are those which are not conflicted with US_ASCII.
+ * So , UCS-2, UCS-4 etc encodings are not supported.
  */
 typedef enum  ml_char_encoding
 {
@@ -72,7 +72,7 @@ typedef enum  ml_char_encoding
 
 	ML_ISO2022CN ,
 	
-	MAX_CHAR_ENCODINGS ,
+	MAX_CHAR_ENCODINGS
 	
 }  ml_char_encoding_t ;
 
