@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_config_menu.c,v 1.6 2003/09/16 03:18:31 h_minami Exp $
+ *	$Id: ml_config_menu.c,v 1.8 2007/04/28 03:39:25 h_minami Exp $
  */
 
 #include  "ml_config_menu.h"
@@ -11,7 +11,7 @@
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_util.h>	/* DIGIT_STR_LEN */
 #include  <kiklib/kik_mem.h>	/* malloc */
-
+#include  <kiklib/kik_file.h>   /* kik_file_set_cloexec, kik_file_unset_cloexec */
 
 #ifndef  LIBEXECDIR
 #define  LIBEXECDIR  /usr/local/libexec
@@ -33,6 +33,7 @@ sig_child(
 	if( config_menu->pid == pid)
 	{
 		config_menu->pid = 0 ;
+		close( config_menu->fd) ;
 		config_menu->fd = -1 ;
 	}
 }
@@ -83,6 +84,12 @@ ml_config_menu_start(
 		return  0 ;
 	}
 
+	if( !kik_file_unset_cloexec( pty_fd))
+	{
+		/* configulators require an inherited pty. */
+		return  0 ;
+	}
+
 	if( pipe( fds) == -1)
 	{
 		return  0 ;
@@ -131,8 +138,14 @@ ml_config_menu_start(
 
 		close( fds[1]) ;
 
-		if( dup2( fds[0] , STDIN_FILENO) == -1 || dup2( pty_fd , STDOUT_FILENO) == -1 ||
-			execv( cmd_path , args) == -1)
+		/* for configulators,
+		 * STDIN => to read replys from mlterm
+		 * STDOUT => to write the "master" side of pty
+		 * STDERR => is retained to be the mlterm's STDERR
+		 */
+		if( dup2( fds[0] , STDIN_FILENO) == -1
+		 || dup2( pty_fd , STDOUT_FILENO) == -1
+	       	 || execv( cmd_path , args) == -1)
 		{
 			/* failed */
 
@@ -148,6 +161,9 @@ ml_config_menu_start(
 
 	config_menu->fd = fds[1] ;
 	config_menu->pid = pid ;
+	
+	kik_file_set_cloexec( pty_fd) ;
+	kik_file_set_cloexec( config_menu->fd) ;
 
 	return  1 ;
 }
