@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_screen.c,v 1.36 2005/03/14 06:33:18 seiichi Exp $
+ *	$Id: ml_screen.c,v 1.37 2006/10/24 10:10:42 h_minami Exp $
  */
 
 #include  "ml_screen.h"
@@ -2106,6 +2106,19 @@ ml_screen_is_cursor_visible(
 	return screen->is_cursor_visible ;
 }
 
+int
+ml_screen_is_alternative_edit(
+	ml_screen_t *  screen
+        )
+{
+	if( screen->edit == &(screen->alt_edit) )
+	{
+		return  1 ;
+	}
+
+	return  0 ;
+}
+
 int
 ml_screen_use_normal_edit(
 	ml_screen_t *  screen
