@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_term.h,v 1.26 2005/02/17 01:59:57 h_minami Exp $
+ *	$Id: ml_term.h,v 1.27 2007/04/26 15:14:13 h_minami Exp $
  */
 
 /*
@@ -101,7 +101,7 @@ int  ml_term_init_encoding_parser( ml_term_t *  term) ;
 
 int  ml_term_init_encoding_conv( ml_term_t *  term) ;
 
-int  ml_term_enable_logging_vt_seq( ml_term_t *  term) ;
+int  ml_term_set_logging_vt_seq( ml_term_t *  term , int  flag) ;
 
 int  ml_term_get_pty_fd( ml_term_t *  term) ;
 
