@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_brightness.c,v 1.9 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id: mc_brightness.c,v 1.12 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_brightness.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -21,8 +21,8 @@
 
 /* --- static variables --- */
 
-static char *  new_brightness ;
-static char *  old_brightness ;
+static char *  new_brightness = NULL;
+static char *  old_brightness = NULL;
 static int is_changed;
 
 
@@ -34,7 +34,8 @@ brightness_selected(
 	gpointer  data
 	)
 {
-	new_brightness = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_brightness);
+	new_brightness = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s brightness is selected.\n" , new_brightness) ;
@@ -65,7 +66,7 @@ mc_brightness_config_widget_new(
 		"10" ,
 	} ;
 
-	old_brightness = new_brightness = mc_get_str_value( "brightness") ;
+	old_brightness = strdup( new_brightness = mc_get_str_value( "brightness"));
 	is_changed = 0;
 	
 	return  mc_combo_new_with_width(_("Brightness"), brightnesses,
