@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_im.c,v 1.17 2004/10/16 06:40:57 seiichi Exp $
+ *	$Id: mc_im.c,v 1.25 2005/03/02 14:13:46 seiichi Exp $
  */
 
 #include "mc_im.h"
 
-#include <string.h>
-#include <stdlib.h>		/* free */
+#include <kiklib/kik_str.h>
+#include <kiklib/kik_mem.h>		/* free */
 #include <kiklib/kik_debug.h>
 #include <kiklib/kik_file.h>
 #include <kiklib/kik_conf_io.h>
@@ -41,19 +41,24 @@
 
 KIK_MAP_TYPEDEF(xim_locale, char*, char*);
 
+#define IM_MAX  20
+
 typedef enum im_type {
-	IM_XIM,
-	IM_NONE,
-	IM_OTHER,
+	IM_NONE = 0,
+	IM_XIM = 1,
+	IM_OTHER = 2,
+
+	IM_TYPE_MAX = IM_MAX
+
 } im_type_t ;
 
 typedef im_info_t* (*im_get_info_func_t)(char *, char *);
 
-#define MAX_IM_INFO 5
 
 /* --- static variables --- */
 
-static im_type_t im_type ;
+static im_type_t im_type;
+static im_type_t cur_im_type;
 
 static char **xims;
 static char **locales;
@@ -66,12 +71,12 @@ static char current_locale_str[STR_LEN] = "";
 static char selected_xim_name[STR_LEN] = "";
 static char selected_xim_locale[STR_LEN] = "";
 
-static im_info_t *im_info_table[MAX_IM_INFO];
+static im_info_t *im_info_table[IM_MAX];
 static u_int  num_of_info = 0;
 static im_info_t *selected_im = NULL;
-static int selected_im_arg = 0;
+static int selected_im_arg = -1 ;
 
-static GtkWidget *im_opt_widget[MAX_IM_INFO];
+static GtkWidget *im_opt_widget[IM_MAX];
 
 /* --- static functions --- */
 
@@ -91,15 +96,15 @@ get_im_info(char *locale, char *encoding)
 {
 	DIR *dir;
 	struct dirent *d;
-	int num = 0;
 
 	if (!(dir = opendir(IM_DIR))) return 0;
 
-	while (d = readdir(dir)) {
+	while ( (d = readdir(dir)) ) {
 		kik_dl_handle_t handle;
-		im_get_info_func_t func ;
+		im_get_info_func_t func;
 		im_info_t *info;
-		char *p ;
+		char symname[100];
+		char *p;
 
 		if (d->d_name[0] == '.' || !is_im_plugin(d->d_name)) continue;
 
@@ -112,8 +117,9 @@ get_im_info(char *locale, char *encoding)
 
 		if (!(handle = kik_dl_open(IM_DIR , p))) continue;
 
-		func = (im_get_info_func_t)kik_dl_func_symbol(handle ,
-							      "im_get_info");
+		snprintf(symname, 100, "im_%s_get_info", &p[3]);
+
+		func = (im_get_info_func_t)kik_dl_func_symbol(handle , symname);
 		if (!func) {
 			kik_dl_close(handle);
 			continue;
@@ -128,6 +134,8 @@ get_im_info(char *locale, char *encoding)
 
 		kik_dl_close(handle);
 	}
+
+	return 0;
 }
 
 
@@ -135,17 +143,6 @@ get_im_info(char *locale, char *encoding)
  * XIM
  */
 
-static gint
-locale_changed(GtkWidget *widget, gpointer data)
-{
-	is_changed = 1;
-
-	snprintf(selected_xim_locale, STR_LEN,
-		 gtk_entry_get_text(GTK_ENTRY(widget)));
-
-	return 1;
-}
-
 static char *
 get_xim_locale(char *xim)
 {
@@ -361,6 +358,9 @@ im_widget_new(int nth_im, const char *value, char *locale)
 		}
 	}
 
+	if (!info->num_of_args)
+		return NULL;
+
 	if (!value || (value && selected)) {
 		char *auto_str;
 
@@ -370,8 +370,7 @@ im_widget_new(int nth_im, const char *value, char *locale)
 		len = strlen(_("auto (currently %s)")) +
 		      strlen(info->readable_args[0]) + 1;
 
-		if (auto_str = malloc(len))
-		{
+		if ((auto_str = malloc(len))) {
 			snprintf(auto_str, len, _("auto (currently %s)"),
 				 info->readable_args[0]);
 			free(info->readable_args[0]);
@@ -385,6 +384,8 @@ im_widget_new(int nth_im, const char *value, char *locale)
 	combo = mc_combo_new(_("Option"), info->readable_args,
 			     info->num_of_args, info->readable_args[selected],
 			     1, im_selected, NULL);
+
+	return combo;
 }
 
 /*
@@ -409,9 +410,9 @@ static gint
 button_im_checked(GtkWidget *widget, gpointer  data)
 {
 	int i;
-	int idx;
+	int idx = 0;
 
-	if (data == NULL) {
+	if (data == NULL || num_of_info == 0) {
 		if (GTK_TOGGLE_BUTTON(widget)->active) {
 			im_type = IM_NONE;
 		}
@@ -421,11 +422,13 @@ button_im_checked(GtkWidget *widget, gpointer  data)
 				idx = i;
 
 		if (GTK_TOGGLE_BUTTON(widget)->active) {
-			im_type = IM_OTHER;
+			im_type = IM_OTHER + idx;
 			selected_im = data;
-			gtk_widget_show(GTK_WIDGET(im_opt_widget[idx]));
+			if (im_opt_widget[idx])
+				gtk_widget_show(GTK_WIDGET(im_opt_widget[idx]));
 		} else {
-			gtk_widget_hide(GTK_WIDGET(im_opt_widget[idx]));
+			if (im_opt_widget[idx])
+				gtk_widget_hide(GTK_WIDGET(im_opt_widget[idx]));
 		}
 	}
 
@@ -447,7 +450,7 @@ mc_im_config_widget_new(void)
 	char *value;
 	char *p;
 	int i;
-	int index;
+	int index = -1;
 	GtkWidget *xim;
 	GtkWidget *frame;
 	GtkWidget *vbox;
@@ -464,22 +467,23 @@ mc_im_config_widget_new(void)
 
 	p = kik_str_sep(&value, ":");
 
+	im_type = IM_NONE ;
 	if (strncmp(p, "xim", 3) == 0) {
 		im_type = IM_XIM;
 		xim_name = kik_str_sep(&value, ":");
 		xim_locale = kik_str_sep(&value, ":");
 	} else if (strncmp(p, "none", 4) == 0) {
-		im_type = IM_NONE;
+		/* do nothing */
 	} else {
-		im_type = IM_NONE;
 		for (i = 0; i < num_of_info; i++) {
 			if (strcmp(p, im_info_table[i]->id) == 0) {
 				index = i;
-				im_type = IM_OTHER;
+				im_type = IM_OTHER + i;
 				break;
 			}
 		}
 	}
+	cur_im_type = im_type ;
 
 	xim = xim_widget_new(xim_name, xim_locale, cur_locale);
 	for (i = 0; i < num_of_info; i++) {
@@ -517,13 +521,13 @@ mc_im_config_widget_new(void)
 				   im_info_table[i]);
 		gtk_widget_show(GTK_WIDGET(radio));
 		gtk_box_pack_start(GTK_BOX(hbox), radio, FALSE, FALSE, 0);
-		if (im_type == IM_OTHER && index == i)
+		if (im_type >= IM_OTHER && index == i)
 			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(radio) ,
 						    TRUE);
 	}
 
 	group = gtk_radio_button_group(GTK_RADIO_BUTTON(radio));
-	radio = gtk_radio_button_new_with_label(group, "None");
+	radio = gtk_radio_button_new_with_label(group, _("None"));
 	gtk_signal_connect(GTK_OBJECT(radio), "toggled",
 			   GTK_SIGNAL_FUNC(button_im_checked), NULL);
 	gtk_widget_show(GTK_WIDGET(radio));
@@ -538,33 +542,40 @@ mc_im_config_widget_new(void)
 	case IM_XIM:
 		gtk_widget_show(xim);
 		for (i = 0; i < num_of_info; i++)
-			gtk_widget_hide(im_opt_widget[i]);
+			if (im_opt_widget[i])
+				gtk_widget_hide(im_opt_widget[i]);
 		break;
 	case IM_NONE:
 		gtk_widget_hide(xim);
 		for (i = 0; i < num_of_info; i++)
-			gtk_widget_hide(im_opt_widget[i]);
+			if (im_opt_widget[i])
+				gtk_widget_hide(im_opt_widget[i]);
 		break;
-	case IM_OTHER:
+	default: /* OTHER */
 		gtk_widget_hide(xim);
 		for (i = 0; i < num_of_info; i++) {
+			if (!im_opt_widget[i])
+				continue;
+
 			if (i == index)
 				gtk_widget_show(im_opt_widget[i]);
 			else
 				gtk_widget_hide(im_opt_widget[i]);
 		}
 		break;
-	default:
-		break;
 	}
 
 	gtk_box_pack_start(GTK_BOX(vbox), xim, TRUE, TRUE, 0);
 	for (i = 0; i < num_of_info; i++)
-		gtk_box_pack_start(GTK_BOX(vbox), im_opt_widget[i], TRUE, TRUE, 0);
+		if (im_opt_widget[i])
+			gtk_box_pack_start(GTK_BOX(vbox), im_opt_widget[i],
+					   TRUE, TRUE, 0);
 
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
+	is_changed = 0 ;
+
 	return frame;
 }
 
@@ -574,7 +585,18 @@ mc_update_im(void)
 	char *p;
 	size_t len;
 
-	if (!is_changed) return;
+	if (!is_changed)
+		return;
+
+	if (im_type == cur_im_type && selected_im_arg == -1) {
+		is_changed = 0;
+		return;
+	}
+
+	if (selected_im_arg == -1) {
+		is_changed = 0;
+		selected_im_arg = 0 ;
+	}
 
 	switch (im_type) {
 	case IM_XIM:
@@ -593,7 +615,8 @@ mc_update_im(void)
 	case IM_NONE:
 		p = strdup("none");
 		break;
-	case IM_OTHER:
+	/* case IM_OTHER: */
+	default:
 		if (selected_im == NULL) return;
 		if (selected_im_arg == 0) { /* auto */
 			p = strdup(selected_im->id);
@@ -604,14 +627,15 @@ mc_update_im(void)
 			sprintf(p, "%s:%s", selected_im->id ,
 				selected_im->args[selected_im_arg]);
 		}
-	default:
 		break;
 	}
 
 	mc_set_str_value("input_method", p);
 
-	free(p);
-
+	selected_im_arg = -1;
+	cur_im_type = im_type;
 	is_changed = 0;
+
+	free(p);
 }
 
