@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_line_space.c,v 1.8 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id: mc_line_space.c,v 1.11 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_line_space.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -21,8 +21,8 @@
 
 /* --- static variables --- */
 
-static char *  new_line_space ;
-static char *  old_line_space ;
+static char *  new_line_space = NULL;
+static char *  old_line_space = NULL;
 static int is_changed;
 
 
@@ -34,7 +34,8 @@ line_space_selected(
 	gpointer  data
 	)
 {
-	new_line_space = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_line_space);
+	new_line_space = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s line_space is selected.\n" , new_line_space) ;
@@ -59,7 +60,7 @@ mc_line_space_config_widget_new(void)
 		"0" ,
 	} ;
 
-	new_line_space = old_line_space = mc_get_str_value( "line_space") ;
+	new_line_space = strdup( old_line_space = mc_get_str_value( "line_space")) ;
 	is_changed = 0;
 
 	return  mc_combo_new_with_width(_("Line space (pixels)"), line_spaces,
