@@ -1,11 +1,11 @@
 /*
- *	$Id: mc_tabsize.c,v 1.11 2003/03/01 14:59:48 arakiken Exp $
+ *	$Id: mc_tabsize.c,v 1.14 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_tabsize.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <glib.h>
 #include  <c_intl.h>
@@ -21,8 +21,8 @@
 
 /* --- static variables --- */
 
-static char *  new_tabsize ;
-static char *  old_tabsize ;
+static char *  new_tabsize = NULL;
+static char *  old_tabsize = NULL;
 static int is_changed;
 
 
@@ -34,7 +34,8 @@ tabsize_selected(
 	gpointer  data
 	)
 {
-	new_tabsize = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_tabsize);
+	new_tabsize = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s tabsize is selected.\n" , new_tabsize) ;
@@ -56,7 +57,7 @@ mc_tabsize_config_widget_new(void)
 		"2" ,
 	} ;
 
-	new_tabsize = old_tabsize = mc_get_str_value( "tabsize") ;
+	new_tabsize = strdup( old_tabsize = mc_get_str_value( "tabsize")) ;
 	is_changed = 0;
 
 	return  mc_combo_new_with_width( _("Tab width (columns)"), tabsizes,
