@@ -1,5 +1,5 @@
 /*
- *	$Id: x_display.c,v 1.4 2003/12/09 11:56:13 h_minami Exp $
+ *	$Id: x_display.c,v 1.5 2006/10/19 14:41:51 h_minami Exp $
  */
 
 #include  "x_display.h"
@@ -7,6 +7,7 @@
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_mem.h>
 #include  <kiklib/kik_str.h>	/* strdup */
+#include  <kiklib/kik_file.h>	/* kik_file_set_cloexec */
 
 #include  "x_xim.h"
 #include  "x_picture.h"
@@ -39,6 +40,9 @@ open_display(
 		goto  error1 ;
 	}
 
+	/* set close-on-exec flag on the socket connected to X. */
+	kik_file_set_cloexec( XConnectionNumber( disp->display));
+
 	if( ( disp->name = strdup( name)) == NULL)
 	{
 		goto  error2 ;
