@@ -1,5 +1,5 @@
 /*
- *	$Id: x_font.c,v 1.20 2004/03/22 12:21:55 h_minami Exp $
+ *	$Id: x_font.c,v 1.21 2006/03/01 10:58:49 seiichi Exp $
  */
 
 #include  "x_font.h"
@@ -524,6 +524,9 @@ set_xft_font(
 
 font_found:
 
+#ifdef FC_EMBOLDEN /* Synthetic emboldening (fontconfig >= 2.3.0) */
+	font->is_double_drawing = 0 ;
+#else
 	if( weight == XFT_WEIGHT_BOLD &&
 		XftPatternGetInteger( xfont->pattern , XFT_WEIGHT , 0 , &weight) == XftResultMatch &&
 		weight != XFT_WEIGHT_BOLD)
@@ -534,6 +537,7 @@ font_found:
 	{
 		font->is_double_drawing = 0 ;
 	}
+#endif
 
 	font->xft_font = xfont ;
 
