@@ -1,14 +1,15 @@
 /*
- *	$Id: x_font.h,v 1.4 2003/09/13 02:07:46 arakiken Exp $
+ *	$Id: x_font.h,v 1.9 2004/12/13 09:31:55 h_minami Exp $
  */
 
 #ifndef  __X_FONT_H__
 #define  __X_FONT_H__
 
 
+/* This must be included ahead of Xft.h on XFree86-4.0.x or before. */
 #include  <X11/Xlib.h>
 
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 #include  <X11/Xft/Xft.h>
 #endif
 
@@ -16,6 +17,7 @@
 #include  <mkf/mkf_charset.h>	/* mkf_charset_t */
 #include  <ml_font.h>
 
+#include  "x_type_engine.h"
 #include  "x_decsp_font.h"
 
 
@@ -25,6 +27,8 @@ typedef enum x_font_present
 	FONT_AA = 0x02 ,
 	FONT_VERTICAL = 0x04 ,
 
+	FONT_PRESENT_MAX
+
 } x_font_present_t ;
 
 typedef struct x_font
@@ -42,10 +46,12 @@ typedef struct x_font
 	int8_t  is_vertical ;
 	int8_t  is_var_col_width ;
 	
-#ifdef  ANTI_ALIAS
+#ifdef  USE_TYPE_XFT
 	XftFont *  xft_font ;
 #endif
+#ifdef  USE_TYPE_XCORE
 	XFontStruct *  xfont ;
+#endif
 	x_decsp_font_t *  decsp_font ;
 
 	/*
@@ -68,8 +74,9 @@ typedef struct x_font
 int  x_compose_dec_special_font(void) ;
 
 
-x_font_t *  x_font_new( Display *  display , ml_font_t  id , x_font_present_t  font_present ,
-	char *  fontname , u_int  fontsize , u_int  col_width , int  use_medium_for_bold) ;
+x_font_t *  x_font_new( Display *  display , ml_font_t  id , x_type_engine_t  type_engine ,
+	x_font_present_t  font_present , char *  fontname , u_int  fontsize ,
+	u_int  col_width , int  use_medium_for_bold) ;
 
 int  x_font_delete( x_font_t *  font) ;
 
@@ -85,6 +92,8 @@ int  x_change_font_cols( x_font_t *  font , u_int  cols) ;
 
 u_int  x_calculate_char_width( x_font_t *  font , u_char *  ch , size_t  len , mkf_charset_t  cs) ;
 
+char **  x_font_get_cs_names( mkf_charset_t  cs) ;
+
 #ifdef  DEBUG
 int  x_font_dump( x_font_t *  font) ;
 #endif
