@@ -1,5 +1,5 @@
 /*
- *	$Id: x_main_config.c,v 1.13 2004/10/10 12:48:44 seiichi Exp $
+ *	$Id: x_main_config.c,v 1.18 2005/02/21 06:51:34 h_minami Exp $
  */
 
 #include  "x_main_config.h"
@@ -176,8 +176,11 @@ x_prepare_for_main_config(
 	kik_conf_add_opt( conf , 'z' ,  "largesmall" , 0 , "step_in_changing_font_size" ,
 		"step in changing font size in GUI configurator [1]") ;
 	kik_conf_add_opt( conf , '\0' , "im" , 0 , "input_method" ,
-		"input method (xim/kbd/uim/iiimf/m17nlib/none) [xim]") ;
-
+		"input method (xim/kbd/uim/iiimf/m17nlib/scim/none) [xim]") ;
+#if defined(USE_IMLIB) || defined(USE_GDK_PIXBUF)
+       kik_conf_add_opt( conf , '\0' , "iconpath" , 0 , "icon_path" ,
+		"path to an imagefile to be use as an window icon") ;
+#endif
 	kik_conf_set_end_opt( conf , 'e' , NULL , "exec_cmd" , 
 		"execute external command") ;
 
@@ -471,7 +474,7 @@ x_main_config_init(
 	{
 		u_int  ratio ;
 
-		if( kik_str_to_uint( &ratio , value))
+		if( kik_str_to_uint( &ratio , value) && ratio)
 		{
 			main_config->screen_width_ratio = ratio ;
 		}
@@ -483,7 +486,7 @@ x_main_config_init(
 	{
 		u_int  ratio ;
 
-		if( kik_str_to_uint( &ratio , value))
+		if( kik_str_to_uint( &ratio , value) && ratio)
 		{
 			main_config->screen_height_ratio = ratio ;
 		}
@@ -907,7 +910,7 @@ x_main_config_init(
 					}
 					else if( *p1 == 'e')
 					{
-						*(p2 ++) = '\e' ;
+						*(p2 ++) = '\033' ;
 					}
 					else
 					{
