@@ -1,5 +1,5 @@
 /*
- *	$Id: x_sb_screen.c,v 1.16 2004/03/05 04:54:27 h_minami Exp $
+ *	$Id: x_sb_screen.c,v 1.17 2005/02/10 15:15:05 h_minami Exp $
  */
 
 #include  "x_sb_screen.h"
@@ -754,10 +754,7 @@ x_sb_screen_new(
 	return  sb_screen ;
 
 error:
-	if( sb_screen)
-	{
-		free( sb_screen) ;
-	}
+	free( sb_screen) ;
 
 	return  NULL ;
 }
