@@ -1,5 +1,5 @@
 /*
- *	$Id: x_selection.h,v 1.2 2003/01/26 11:26:59 arakiken Exp $
+ *	$Id: x_selection.h,v 1.3 2004/02/22 07:49:48 arakiken Exp $
  */
 
 #ifndef  __X_SELECTION_H__
@@ -7,7 +7,7 @@
 
 
 #include  <kiklib/kik_types.h>		/* u_int */
-#include  <ml_char.h>
+#include  <ml_str.h>
 
 
 typedef struct  x_sel_event_listener
