@@ -1,5 +1,5 @@
 /*
- *	$Id: x_shortcut.c,v 1.12 2004/12/13 09:31:55 h_minami Exp $
+ *	$Id: x_shortcut.c,v 1.13 2006/10/13 15:29:10 h_minami Exp $
  */
 
 #include  "x_shortcut.h"
@@ -103,7 +103,7 @@ parse(
 
 		map_entry = shortcut->map + key_func_table[count].func ;
 		if( (map_entry->ksym == ksym) &&
-		    (map_entry->state = state))
+		    (map_entry->state == state))
 		{
 			map_entry->is_used = 0 ;
 		}
