@@ -1,5 +1,5 @@
 /*
- *	$Id: x_term_manager.c,v 1.92 2006/01/08 11:46:42 kzys Exp $
+ *	$Id: x_term_manager.c,v 1.95 2006/11/20 11:22:34 seiichi Exp $
  */
 
 #include  "x_term_manager.h"
@@ -210,14 +210,14 @@ open_pty_intern(
 	*(env_p ++) = wid_env ;
 	
 	/* "DISPLAY="(8) + NULL(1) */
-	if( ( disp_env = alloca( 8 + strlen( display) + 1)))
+	if( display && ( disp_env = alloca( 8 + strlen( display) + 1)))
 	{
 		sprintf( disp_env , "DISPLAY=%s" , display) ;
 		*(env_p ++) = disp_env ;
 	}
 
 	/* "TERM="(5) + NULL(1) */
-	if( ( term_env = alloca( 5 + strlen( term_type) + 1)))
+	if( term_type && ( term_env = alloca( 5 + strlen( term_type) + 1)))
 	{
 		sprintf( term_env , "TERM=%s" , term_type) ;
 		*(env_p ++) = term_env ;
@@ -872,7 +872,8 @@ start_daemon(void)
 	{
 		return  -1 ;
 	}
-
+	kik_file_set_cloexec( sock_fd);
+	
 	while( bind( sock_fd , (struct sockaddr *) &servaddr , sizeof( servaddr)) < 0)
 	{
 		if( errno == EADDRINUSE)
@@ -993,7 +994,7 @@ client_connected(void)
 	 * If this flag off, this fd remained open until the child process forked in
 	 * open_screen_intern()(ml_term_open_pty()) close it.
 	 */
-	fcntl( fd , F_SETFD , 1) ;
+	kik_file_set_cloexec( fd) ;
 
 	if( ( fp = fdopen( fd , "r+")) == NULL)
 	{
@@ -1753,6 +1754,12 @@ void
 x_term_manager_event_loop(void)
 {
 	int  count ;
+	const char  * display ;
+
+	if( ! *( display = main_config.disp_name) && ( ! ( display = getenv( "DISPLAY")) || ! *display))
+	{
+		display = ":0.0" ;
+	}
 
 	for( count = 0 ; count < num_of_startup_screens ; count ++)
 	{
@@ -1785,7 +1792,7 @@ x_term_manager_event_loop(void)
 		}
 
 		if( ! open_pty_intern( term , main_config.cmd_path , main_config.cmd_argv ,
-			":0.0" , 0 , main_config.term_type , main_config.use_login_shell))
+			display , 0 , main_config.term_type , main_config.use_login_shell))
 		{
 			return ;
 		}
@@ -1849,6 +1856,7 @@ x_term_manager_add_fd(
 		{
 			additional_fds[i].fd = fd ;
 			additional_fds[i].handler = handler ;
+			kik_file_set_cloexec( fd);
 
 			return  0 ;
 		}
