@@ -1,5 +1,5 @@
 /*
- *	$Id: x_termcap.c,v 1.5 2002/09/06 11:38:55 h_minami Exp $
+ *	$Id: x_termcap.c,v 1.6 2003/10/11 12:23:05 arakiken Exp $
  */
 
 #include  "x_termcap.h"
@@ -281,7 +281,7 @@ x_termcap_final(
 }
 
 int
-x_termcap_read_conf(
+x_read_termcap_config(
 	x_termcap_t *  termcap ,
 	char *  filename
 	)
