@@ -1,5 +1,5 @@
 /*
- *	$Id: x_window_manager.c,v 1.7 2003/12/25 12:18:17 h_minami Exp $
+ *	$Id: x_window_manager.c,v 1.10 2005/02/21 14:43:43 h_minami Exp $
  */
 
 #include  "x_window_manager.h"
@@ -85,9 +85,10 @@ x_window_manager_init(
 	win_man->group_leader = XCreateSimpleWindow( win_man->display,
 						     win_man->my_window,
 						     0, 0, 1, 1, 0, 0, 0) ;
+	win_man->icon_path = NULL;
 	win_man->icon = None ;
 	win_man->mask = None ;
-	win_man->cardinal = 0 ;
+	win_man->cardinal = NULL ;
 
 	win_man->roots = NULL ;
 	win_man->num_of_roots = 0 ;
@@ -117,6 +118,9 @@ x_window_manager_final(
 	{
 		XDestroyWindow( win_man->display, win_man->group_leader) ;
 	}
+
+	free(  win_man->icon_path);
+
 	if( win_man->icon)
 	{
 		XFreePixmap( win_man->display, win_man->icon) ;
@@ -125,20 +129,16 @@ x_window_manager_final(
 	{
 		XFreePixmap( win_man->display, win_man->mask) ;
 	}
-	if( win_man->cardinal)
-	{
-		free( win_man->cardinal) ;
-	}
+
+	free( win_man->cardinal) ;
+
 	for( count = 0 ; count < win_man->num_of_roots ; count ++)
 	{
 		x_window_unmap( win_man->roots[count]) ;
 		x_window_final( win_man->roots[count]) ;
 	}
 
-	if( win_man->roots)
-	{
-		free( win_man->roots) ;
-	}
+	free( win_man->roots) ;
 
 	return  1 ;
 }
@@ -338,3 +338,56 @@ x_window_manager_update_modifier_mapping(
 		win_man->modmap.serial = serial ;
 	}
 }
+
+int x_window_manager_set_icon(
+	x_window_t *  win,
+	char *  icon_path
+	)
+{
+	x_window_manager_t *  win_man ;
+
+	win_man = win->win_man ;
+
+	if( !icon_path && !win_man->icon_path)
+	{
+		/* dont't need icon at all? */
+		return  0 ;
+	}
+
+	if( !win_man->icon_path)
+	{
+		x_window_t  dummy = {0};
+
+		/* register the default icon */
+		if(!(win_man->icon_path = strdup( icon_path)))
+		{
+			return  0 ;
+		}
+		dummy.my_window = win_man->group_leader ;
+		dummy.display = win_man->display ;
+		dummy.icon_pix = None ;
+		dummy.icon_mask = None ;
+		dummy.icon_card = NULL ;
+		x_window_set_icon_from_file( &dummy, icon_path);
+
+		win_man->icon = dummy.icon_pix ;
+		win_man->mask = dummy.icon_mask ;
+		win_man->cardinal = dummy.icon_card ;
+
+	}
+
+	if( !icon_path || strcmp( icon_path, win_man->icon_path) ==0)
+	{
+		x_window_remove_icon( win) ;
+		/* use a default icon from window manager */
+		return x_window_set_icon( win,
+					  win_man->icon,
+					  win_man->mask,
+					  win_man->cardinal) ;
+	}
+	else
+	{
+		/* load new icon from "icon_path" */
+		return x_window_set_icon_from_file( win, icon_path);
+	}
+}
