@@ -1,5 +1,5 @@
 /*
- *	$Id: x_window_manager.h,v 1.5 2003/06/22 11:19:38 h_minami Exp $
+ *	$Id: x_window_manager.h,v 1.8 2005/02/21 14:43:43 h_minami Exp $
  */
 
 #ifndef  __X_WINDOW_MANAGER_H__
@@ -11,6 +11,13 @@
 #include  "x_window.h"
 
 
+typedef struct  x_modifier_mapping
+{
+	u_long  serial ;
+	XModifierKeymap *  map ;
+
+} x_modifier_mapping_t ;
+
 typedef struct  x_window_manager
 {
 	Display *  display ;
@@ -18,6 +25,7 @@ typedef struct  x_window_manager
 	Window  my_window ;
 
 	Window group_leader ;
+	char *icon_path;
 	Pixmap  icon ;
 	Pixmap  mask ;
 	u_int32_t *  cardinal ;
@@ -26,7 +34,8 @@ typedef struct  x_window_manager
 	u_int  num_of_roots ;
 
 	x_window_t *  selection_owner ;
-	
+	x_modifier_mapping_t  modmap ;
+
 } x_window_manager_t ;
 
 
@@ -49,5 +58,9 @@ int  x_window_manager_receive_next_event( x_window_manager_t *  win_man) ;
 
 XID  x_window_manager_get_group( x_window_manager_t *  win_man) ;
 
+XModifierKeymap *  x_window_manager_get_modifier_mapping( x_window_manager_t *  win_man) ;
 
+void  x_window_manager_update_modifier_mapping( x_window_manager_t *  win_man ,	u_int  serial) ;
+
+int x_window_manager_set_icon( x_window_t *  win , char *  icon_path) ;
 #endif
