@@ -1,11 +1,13 @@
 /*
- *	$Id: x_xim.c,v 1.9 2005/02/10 15:15:08 h_minami Exp $
+ *	$Id: x_xim.c,v 1.11 2006/10/23 13:09:36 h_minami Exp $
  */
 
 #include  "x_xim.h"
 
 #include  <stdio.h>		/* sprintf */
 #include  <string.h>		/* strcmp/memset */
+#include  <unistd.h>	 	/* dup/close */
+#include  <kiklib/kik_file.h>	/* kik_set_file_cloexec */
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_str.h>	/* strdup */
 #include  <kiklib/kik_locale.h>	/* kik_locale_init/kik_get_locale/kik_get_codeset */
@@ -121,6 +123,7 @@ open_xim(
 	char *  xmod ;
 	char *  cur_locale ;
 	int  result ;
+	int  next_fd ; /* to deal with brain-dead XIM implemantations */
 
 	/* 4 is the length of "@im=" */
 	if( ( xmod = alloca( 4 + strlen( xim->name) + 1)) == NULL)
@@ -154,6 +157,12 @@ open_xim(
 
 	result = 0 ;
 
+	next_fd = dup( 0) ;
+	if( next_fd != -1)
+	{
+		/* remember the lowest unused fd */
+		close( next_fd) ;
+	}	
 	if( XSetLocaleModifiers(xmod) && ( xim->im = XOpenIM( display , NULL , NULL , NULL)))
 	{
 		if( ( xim->encoding = ml_get_char_encoding( kik_get_codeset())) == ML_UNKNOWN_ENCODING ||
@@ -172,7 +181,12 @@ open_xim(
 			result = 1 ;
 		}
 	}
-	
+	if( next_fd > 0)
+	{
+		/* if XOpenIM() internally opens a fd,
+		 * we should close it on exec() */
+		kik_file_set_cloexec( next_fd) ;
+	}
 	if( cur_locale)
 	{
 		/* restoring */
